/* Copyright © 2012 Matthew Champion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
 * Neither the name of mattunderscore.com nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.filter;

import java.io.IOException;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;

/**
 * A HTTP filter that provides boiler plate for casting {@link ServletRequest} to
 * {@link HTTPServeletRequests} etc. If the request or response cannot be cast to the correct
 * classes a servlet exception will be thrown. Provides a
 * {@link javax.servlet.Filter#doHTTPFilter(ServletRequest,ServletResponse,FilterChain)} method that
 * is intended to be implemented to provide the behaviour of the filter.
 * 
 * @author Matt Champion
 * @since 0.0.13
 */
public abstract class GenericHTTPFilter extends GenericFilter
{

    @Override
    public final void doFilter(final ServletRequest request, final ServletResponse response,
            final FilterChain chain) throws IOException, ServletException
    {
        if (response instanceof HttpServletResponse)
        {
            if (request instanceof HttpServletRequest)
            {
                final HttpServletResponse resp = (HttpServletResponse) response;
                final HttpServletRequest req = (HttpServletRequest) request;
                doHTTPFilter(req, resp, chain);
            }
            else
            {
                throw new ServletException("HTTP Filter was passed a "
                        + "non-HttpServletRequest object");
            }
        }
        else
        {
            throw new ServletException("HTTP Filter was passed a non-HttpServletResponse object");
        }
    }

    /**
     * A method that executes the filter behaviour for HTTP requests and responses.
     * 
     * @param request
     *            The HTTP request
     * @param response
     *            The HTTP response
     * @param chain
     *            The filter chain
     * @throws IOException
     * @throws ServletException
     * @since 0.0.13
     */
    public abstract void doHTTPFilter(HttpServletRequest request, HttpServletResponse response,
            FilterChain chain) throws IOException, ServletException;
}
